#include "StdAfx.h"
#include "NtalkLog.h"

#include <log4cxx/basicconfigurator.h>
#include <log4cxx/propertyconfigurator.h>
#include <log4cxx/helpers/exception.h>
#include <log4cxx/patternlayout.h>
#include <log4cxx/rollingfileappender.h>

#define LOG_BUFFER_SIZE	1024

CNtalkLog::CNtalkLog(void)
{
}

CNtalkLog::~CNtalkLog(void)
{
    BasicConfigurator::resetConfiguration();
}

bool CNtalkLog::WriteBinaryToFile(BYTE *pBuffer, DWORD nSize, CString strFilePath)
{
	HANDLE hFile = ::CreateFile(strFilePath, GENERIC_WRITE, FILE_SHARE_READ, NULL, CREATE_ALWAYS , 0, NULL);
	if( hFile != INVALID_HANDLE_VALUE )
	{
		CFile file( hFile );
		file.SetFilePath( strFilePath );

		file.Write(pBuffer, nSize);
		file.Flush();

		return true;
	} 

	return false;
}

bool CNtalkLog::LoadConfigurationFromRes( CString strLogConfigFilePath )
{
	bool bRsult = false;

	HINSTANCE hInstance = AfxGetResourceHandle();

	HRSRC hResInfo = FindResource(hInstance, MAKEINTRESOURCE(IDR_CONFIGURATION_LOG4CXX), _T("CONFIGURATION"));
	HANDLE hRes = LoadResource(hInstance , hResInfo);;

	int nSize = 0;
	LPSTR lpRes = (char*)LockResource(hRes);
	if (lpRes != NULL)
	{
		nSize = SizeofResource(hInstance , hResInfo);

		BYTE* pBuff = NULL;
		pBuff = new BYTE[nSize];
		memcpy(pBuff, lpRes, nSize);

		bRsult = WriteBinaryToFile(pBuff, nSize, strLogConfigFilePath);

		delete [] pBuff;

		UnlockResource(hRes);
	}

	FreeResource(hRes);

	return bRsult;
}

void CNtalkLog::Initialize(void)
{
	PatternLayout * pPatternLayout = new PatternLayout();

	LogString conversionPattern = _T("%d{yyyy/MM/dd HH:mm:ss:SSS} (%t) %-5p %m%n");
	pPatternLayout->setConversionPattern(conversionPattern);

	LogString logFileName = GetMyDocNtalkDir() + _T("\\Logs\\ntalk.log");
	RollingFileAppender * fileAppender = new RollingFileAppender(LayoutPtr(pPatternLayout), logFileName, true);

	fileAppender->setEncoding(LOG4CXX_STR("UTF-8"));
	fileAppender->setName(LOG4CXX_STR("localfilelog"));
	fileAppender->setMaxFileSize(LOG4CXX_STR("1MB"));
	fileAppender->setMaxBackupIndex(10);

	Pool p;
	fileAppender->activateOptions(p);

	BasicConfigurator::configure(AppenderPtr(fileAppender));

	Logger::getRootLogger()->setLevel(Level::getDebug());

	rootlog = Logger::getRootLogger();
	logLocalFile = Logger::getLogger("localfilelog");
}

void CNtalkLog::Debug(const char* format, ...)
{
	va_list args;
	va_start(args, format);

	char buf[LOG_BUFFER_SIZE] = {0,};
    vsnprintf(buf, LOG_BUFFER_SIZE - 1, format, args);

	LOG4CXX_DEBUG(logLocalFile, buf);

	va_end(args);
}

void CNtalkLog::Debug(const wchar_t* format, ...)
{
	va_list args;
	va_start(args, format);

	wchar_t buf[LOG_BUFFER_SIZE] = {0,};
    vswprintf(buf, LOG_BUFFER_SIZE - 1, format, args);
	LOG4CXX_DEBUG(logLocalFile, buf);

	va_end(args);
}

// TODO: Refactoring duplicated code
void CNtalkLog::Error(const char* format, ...)
{
	va_list args;
	va_start(args, format);

	char buf[LOG_BUFFER_SIZE] = {0,};
    vsnprintf(buf, LOG_BUFFER_SIZE - 1, format, args);

	LOG4CXX_ERROR(logLocalFile, buf);

	va_end(args);
}

// TODO: Refactoring duplicated code
void CNtalkLog::Error(const wchar_t* format, ...)
{
	va_list args;
	va_start(args, format);

	wchar_t buf[LOG_BUFFER_SIZE] = {0,};
    vswprintf(buf, LOG_BUFFER_SIZE - 1, format, args);

	LOG4CXX_ERROR(logLocalFile, buf);

	va_end(args);
}
